// src/collapsible.anatomy.ts
import { createAnatomy } from "@zag-js/anatomy";
var anatomy = createAnatomy("collapsible").parts("root", "trigger", "content");
var parts = anatomy.build();

// src/collapsible.connect.ts
import { dataAttr } from "@zag-js/dom-query";

// src/collapsible.dom.ts
import { createScope } from "@zag-js/dom-query";
var dom = createScope({
  getRootId: (ctx) => ctx.ids?.root ?? `collapsible:${ctx.id}`,
  getContentId: (ctx) => ctx.ids?.content ?? `collapsible:${ctx.id}:content`,
  getTriggerId: (ctx) => ctx.ids?.trigger ?? `collapsible:${ctx.id}:trigger`,
  getRootEl: (ctx) => dom.getById(ctx, dom.getRootId(ctx)),
  getContentEl: (ctx) => dom.getById(ctx, dom.getContentId(ctx)),
  getTriggerEl: (ctx) => dom.getById(ctx, dom.getTriggerId(ctx))
});

// src/collapsible.connect.ts
function connect(state, send, normalize) {
  const isVisible = state.matches("open", "closing");
  const isOpen = state.matches("open");
  const height = state.context.height;
  const width = state.context.width;
  const isDisabled = !!state.context.disabled;
  const skipMountAnimation = state.context.isMountAnimationPrevented && isOpen;
  return {
    isDisabled,
    isVisible,
    isOpen,
    open() {
      send("OPEN");
    },
    close() {
      send("CLOSE");
    },
    rootProps: normalize.element({
      ...parts.root.attrs,
      "data-state": isOpen ? "open" : "closed",
      dir: state.context.dir,
      id: dom.getRootId(state.context)
    }),
    contentProps: normalize.element({
      ...parts.content.attrs,
      "data-state": skipMountAnimation ? void 0 : isOpen ? "open" : "closed",
      id: dom.getContentId(state.context),
      "data-disabled": dataAttr(isDisabled),
      hidden: !isVisible,
      style: {
        "--height": height != null ? `${height}px` : void 0,
        "--width": width != null ? `${width}px` : void 0
      }
    }),
    triggerProps: normalize.element({
      ...parts.trigger.attrs,
      id: dom.getTriggerId(state.context),
      dir: state.context.dir,
      type: "button",
      "data-state": isOpen ? "open" : "closed",
      "data-disabled": dataAttr(isDisabled),
      "aria-controls": dom.getContentId(state.context),
      "aria-expanded": isVisible || false,
      onClick() {
        if (isDisabled)
          return;
        send({ type: isOpen ? "CLOSE" : "OPEN", src: "trigger.click" });
      }
    })
  };
}

// src/collapsible.machine.ts
import { createMachine, ref } from "@zag-js/core";
import { getComputedStyle, raf } from "@zag-js/dom-query";
import { compact } from "@zag-js/utils";
function machine(userContext) {
  const ctx = compact(userContext);
  return createMachine(
    {
      id: "collapsible",
      initial: ctx.open ? "open" : "closed",
      context: {
        ...ctx,
        height: 0,
        width: 0,
        isMountAnimationPrevented: !!ctx.open,
        stylesRef: null
      },
      watch: {
        open: ["allowAnimation", "toggleVisibility"]
      },
      entry: ["computeSize"],
      states: {
        closed: {
          tags: ["closed"],
          entry: ["computeSize"],
          on: {
            "CONTROLLED.OPEN": {
              target: "open",
              actions: ["computeSize"]
            },
            OPEN: [
              {
                guard: "isOpenControlled",
                actions: ["invokeOnOpen"]
              },
              {
                target: "open",
                actions: ["allowAnimation", "invokeOnOpen"]
              }
            ]
          }
        },
        closing: {
          tags: ["open"],
          activities: ["trackAnimationEvents"],
          on: {
            "CONTROLLED.CLOSE": {
              target: "closed",
              actions: ["invokeOnExitComplete"]
            },
            "CONTROLLED.OPEN": "open",
            OPEN: [
              {
                guard: "isOpenControlled",
                actions: ["invokeOnOpen"]
              },
              {
                target: "open",
                actions: ["allowAnimation", "invokeOnOpen"]
              }
            ],
            CLOSE: [
              {
                guard: "isOpenControlled",
                actions: ["invokeOnClose"]
              },
              {
                target: "closed",
                actions: ["allowAnimation", "computeSize", "invokeOnExitComplete"]
              }
            ],
            "ANIMATION.END": {
              target: "closed",
              actions: ["invokeOnExitComplete"]
            }
          }
        },
        open: {
          tags: ["open"],
          on: {
            "CONTROLLED.CLOSE": {
              target: "closing",
              actions: ["computeSize"]
            },
            CLOSE: [
              {
                guard: "isOpenControlled",
                actions: ["invokeOnClose"]
              },
              {
                target: "closing",
                actions: ["allowAnimation", "computeSize"]
              }
            ]
          }
        }
      }
    },
    {
      guards: {
        isOpenControlled: (ctx2) => !!ctx2["open.controlled"]
      },
      activities: {
        trackAnimationEvents(ctx2, _evt, { send }) {
          let cleanup;
          const rafCleanup = raf(() => {
            const contentEl = dom.getContentEl(ctx2);
            if (!contentEl)
              return;
            const animationName = getComputedStyle(contentEl).animationName;
            const hasNoAnimation = !animationName || animationName === "none";
            if (hasNoAnimation) {
              send({ type: "ANIMATION.END" });
              return;
            }
            const onEnd = (event) => {
              if (event.target !== contentEl)
                return;
              send({ type: "ANIMATION.END" });
            };
            contentEl.addEventListener("animationend", onEnd);
            contentEl.addEventListener("animationcancel", onEnd);
            cleanup = () => {
              contentEl.removeEventListener("animationend", onEnd);
              contentEl.removeEventListener("animationcancel", onEnd);
            };
          });
          return () => {
            rafCleanup();
            cleanup?.();
          };
        }
      },
      actions: {
        allowAnimation(ctx2) {
          ctx2.isMountAnimationPrevented = false;
        },
        computeSize: (ctx2) => {
          raf(() => {
            const contentEl = dom.getContentEl(ctx2);
            if (!contentEl)
              return;
            ctx2.stylesRef || (ctx2.stylesRef = ref({
              animationName: contentEl.style.animationName,
              animationDuration: contentEl.style.animationDuration
            }));
            const hidden = contentEl.hidden;
            contentEl.style.animationName = "none";
            contentEl.style.animationDuration = "0s";
            contentEl.hidden = false;
            const rect = contentEl.getBoundingClientRect();
            ctx2.height = rect.height;
            ctx2.width = rect.width;
            if (!ctx2.isMountAnimationPrevented) {
              contentEl.style.animationName = ctx2.stylesRef.animationName;
              contentEl.style.animationDuration = ctx2.stylesRef.animationDuration;
            }
            contentEl.hidden = hidden;
          });
        },
        invokeOnOpen: (ctx2) => {
          ctx2.onOpenChange?.({ open: true });
        },
        invokeOnClose: (ctx2) => {
          ctx2.onOpenChange?.({ open: false });
        },
        invokeOnExitComplete(ctx2) {
          ctx2.onExitComplete?.();
        },
        toggleVisibility: (ctx2, _evt, { send }) => {
          send({ type: ctx2.open ? "CONTROLLED.OPEN" : "CONTROLLED.CLOSE" });
        }
      }
    }
  );
}

// src/collapsible.props.ts
import { createProps } from "@zag-js/types";
import { createSplitProps } from "@zag-js/utils";
var props = createProps()([
  "dir",
  "disabled",
  "getRootNode",
  "id",
  "ids",
  "onExitComplete",
  "onOpenChange",
  "open.controlled",
  "open"
]);
var splitProps = createSplitProps(props);
export {
  anatomy,
  connect,
  machine,
  props,
  splitProps
};
//# sourceMappingURL=index.mjs.map